@@ -6,8 +6,9 @@ import re
 from datetime import date, datetime
 from pathlib import Path
 from textwrap import dedent
+from typing import Any
 
-from PIL import ExifTags, Image
+from PIL import Image
 from rich.logging import RichHandler
 
 from pwiki import wgen
@@ -15,13 +16,24 @@ from pwiki.wiki import Wiki
 
 log = logging.getLogger(__name__)
 
-MTC_FILE = Path.home() / ".scu.px.txt"
+_MTC_FILE = Path.home() / ".scu.px.txt"
 
 
-def _main():
+def _make_wiki(*args: Any) -> Wiki:
+    """Convienence method, creates a new `Wiki` pointed at Commons with `args`
+
+    Returns:
+        Wiki: The Wiki resulting object
+    """
+    return Wiki("commons.wikimedia.org", *args)
+
+
+def _main() -> None:
+    """Main driver, to be run if this script is invoked directly."""
+
     for lg in (logging.getLogger("pwiki"), log):
         lg.addHandler(RichHandler(rich_tracebacks=True))
-        lg.setLevel("INFO")
+        lg.setLevel(logging.INFO)
 
     cli_parser = argparse.ArgumentParser(description="Simple Commons Uploader")
     cli_parser.add_argument('--user', type=str, help="username to use")
@@ -32,19 +44,18 @@ def _main():
     args = cli_parser.parse_args()
 
     if args.wgen:
-        wgen.setup_px(MTC_FILE, False)
+        wgen.setup_px(_MTC_FILE, False)
         return
 
-    wiki = Wiki("commons.wikimedia.org")
     if args.i:
-        wiki.login(input("Please login to continue.\nUsername: "), getpass.getpass())
+        wiki = _make_wiki(input("Please login to continue.\nUsername: "), getpass.getpass())
     elif args.user:
         if not args.pw:
             log.critical("No password specified, please pass the --pw flag with a pasword.")
             return
-        wiki.login(args.user, args.pw)
-    elif MTC_FILE.is_file():
-        wiki.login(*wgen.load_px(MTC_FILE).popitem())
+        wiki = _make_wiki(args.user, args.pw)
+    elif _MTC_FILE.is_file():
+        wiki = _make_wiki(*wgen.load_px(_MTC_FILE).popitem())
     else:
         cli_parser.print_help()
         return
@@ -62,24 +73,30 @@ def _main():
 
         i = 1
         for f in base_dir.iterdir():
-            ext = f.suffix.lower()
-            if not f.is_file() or ext not in ext_list:
+            if not f.is_file() or (ext := f.suffix.lower()) not in ext_list:
                 continue
 
+            # date
             timestamp = None
             if ext in (".jpg", ".jpeg"):
                 try:
                     with Image.open(f) as img:
-                        if dto := next((v for k, v in img.getexif().items() if ExifTags.TAGS.get(k) == "DateTimeOriginal" and re.match(r"\d{4}:\d{2}:\d{2} \d{2}:\d{2}:\d{2}", v)), None):
+                        if (dto := img.getexif().get(306)) and re.match(r"\d{4}:\d{2}:\d{2} \d{2}:\d{2}:\d{2}", dto):
                             d, t = dto.split()
                             timestamp = f"{d.replace(':', '-')} {t}"
                 except Exception as e:
-                    log.warn("Could not parse EXIF for %s", f, exc_info=True)
+                    log.warning("Could not parse EXIF for %s", f, exc_info=True)
 
-            desc = f"""\
+            # base_desc, cat
+            if "_" in base_dir.name:
+                b, cat = [s.strip() for s in base_dir.name.split("_", 1)]
+            else:
+                b = cat = base_dir.name
+
+            desc = dedent(f"""\
             =={{{{int:filedesc}}}}==
             {{{{Information
-            |description={base_dir.name}
+            |description={b}
             |date={timestamp or datetime.fromtimestamp(f.stat().st_mtime).strftime('%Y-%m-%d %H:%M:%S')}
             |source={{{{Own}}}}
             |author=~~~
@@ -88,18 +105,20 @@ def _main():
             =={{{{int:license-header}}}}==
             {{{{Self|Cc-by-sa-4.0}}}}
             
-            [[Category:{base_dir.name}]]
-            [[Category:Files by {wiki.username}]]"""
+            [[Category:{cat}]]
+            [[Category:Files by {wiki.username}]]""")
 
-            if not wiki.upload(f, f"{base_dir.name} {i} {date.today()}{ext}", dedent(desc)):
+            if not wiki.upload(f, f"{b} {i} {date.today()}{ext}", desc):
                 fails.append(f)
             i += 1
 
     if fails:
-        log.warn("Failed to upload %d files: %s", len(fails), fails)
+        log.warning("Failed to upload %d files: %s", len(fails), fails)
     else:
         log.info("Finished with no failures")
 
+    wiki.save_cookies()
+
 
 if __name__ == '__main__':
     _main()
